<?php

/**
 * Hostapd class.
 *
 * @category   apps
 * @package    wireless_ap
 * @subpackage libraries
 * @author     Todd Lewis <todd@grizzlystar.com> 
 * @copyright  2019 Todd Lewis
 * @license    GPLv3
 */

///////////////////////////////////////////////////////////////////////////////
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// N A M E S P A C E
///////////////////////////////////////////////////////////////////////////////

namespace clearos\apps\wireless_ap;

///////////////////////////////////////////////////////////////////////////////
// B O O T S T R A P
///////////////////////////////////////////////////////////////////////////////

$bootstrap = getenv('CLEAROS_BOOTSTRAP') ? getenv('CLEAROS_BOOTSTRAP') : '/usr/clearos/framework/shared';
require_once $bootstrap . '/bootstrap.php';

///////////////////////////////////////////////////////////////////////////////
// T R A N S L A T I O N S
///////////////////////////////////////////////////////////////////////////////

clearos_load_language('wireless_ap');
clearos_load_language('base');

///////////////////////////////////////////////////////////////////////////////
// D E P E N D E N C I E S
///////////////////////////////////////////////////////////////////////////////

// Classes
//--------

use \clearos\apps\base\Configuration_File as Configuration_File;
use \clearos\apps\base\Daemon as Daemon;
use \clearos\apps\base\File as File;
use \clearos\apps\radius\FreeRADIUS as FreeRADIUS;
use \clearos\apps\network\Iface_Manager as Iface_Manager;

clearos_load_library('base/Configuration_File');
clearos_load_library('base/Daemon');
clearos_load_library('base/Engine');
clearos_load_library('base/File');
clearos_load_library('base/Folder');
clearos_load_library('base/Shell');
clearos_load_library('radius/FreeRADIUS');
clearos_load_library('network/Iface_Manager');

// Exceptions
//-----------

use \clearos\apps\base\File_Not_Found_Exception as File_Not_Found_Exception;
use \clearos\apps\base\Validation_Exception as Validation_Exception;

clearos_load_library('base/File_Not_Found_Exception');
clearos_load_library('base/Validation_Exception');

///////////////////////////////////////////////////////////////////////////////
// C L A S S
///////////////////////////////////////////////////////////////////////////////

/**
 * Hostapd class.
 *
 * @category   apps
 * @package    wireless_ap
 * @subpackage libraries
 * @author     Todd Lewis <todd@grizzlystar.com> 
 * @copyright  2019 Todd Lewis
 * @license    GPLv3
 */

class Hostapd extends Daemon
{
    ///////////////////////////////////////////////////////////////////////////////
    // C O N S T A N T S
    ///////////////////////////////////////////////////////////////////////////////
    
    const COMMAND_IW            = '/usr/sbin/iw';
    const FILE_HOSTAPD_CONFIG   = '/etc/hostapd/hostapd-';
    const MODE_WPA_PSK          = 'WPA-PSK';
    const MODE_WPA_EAP          = 'WPA-EAP';
  

    ///////////////////////////////////////////////////////////////////////////////
    // V A R I A B L E S
    ///////////////////////////////////////////////////////////////////////////////

    protected $is_loaded = FALSE;
    protected $config = array();

    ///////////////////////////////////////////////////////////////////////////////
    // M E T H O D S
    ///////////////////////////////////////////////////////////////////////////////

    public function __construct(){
        clearos_profile(__METHOD__, __LINE__);
    
        parent::__construct('hostapd');
    }

    public function getBetween($content,$start,$end){
        $r = explode($start, $content);
        if (isset($r[1])){
            $r = explode($end, $r[1]);
            return $r[0];
        }
        return '';
    }

    public function random_password($length){
        $chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()_-=+;:,.?";
        $password = substr( str_shuffle( $chars ), 0, $length );
        return $password;
    }

    public function get_wifi_devices(){
        clearos_profile(__METHOD__, __LINE__);

        $output = shell_exec( self::COMMAND_IW . " dev | grep Interface | grep -o -P '(?<=Interface ).*'");
        $devices = preg_split('/\s+/', trim($output));
            
        return $devices;
    }
    
    public function get_device_network_settings($device){
        clearos_profile(__METHOD__, __LINE__);

        $device_network_config = "/etc/sysconfig/network-scripts/ifcfg-$device" ;

        $output = shell_exec(" cat $device_network_config | grep  -oP '(?<=DEVICE=).*' ");

        if (isset($output)){
            $device_network_settings['network_configured'] = TRUE;
            $output =  parse_ini_file($device_network_config);
            if (isset($output)){
                $device_network_settings['type']        = $output[TYPE];
                $device_network_settings['ipaddr']      = $output[IPADDR];
                $device_network_settings['bootproto']   = $output[BOOTPROTO];;
                $device_network_settings['bridge']      = $output[BRIDGE];

                if( $device_network_settings['bootproto'] == 'static')
                    $device_network_settings['network_type'] = lang('wireless_ap_network_type_static') . $device_network_settings['ipaddr'];
                    
                if( $device_network_settings['bootproto'] == 'dhcp')
                       $device_network_settings['network_type'] = lang('wireless_ap_network_type_dhcp');

                if( $device_network_settings['type'] == 'BridgeChild')
                    $device_network_settings['network_type'] = lang('wireless_ap_network_type_bridged') .  $device_network_settings['bridge'] ;
            }
        } else {
            $device_network_settings['network_configured'] = FALSE;
        }

        return $device_network_settings;
    }

    public function get_device_hostapd_settings($device){
        clearos_profile(__METHOD__, __LINE__);

        $hostapd_config = self::FILE_HOSTAPD_CONFIG . $device . '.conf';

        $hostapd_settings['hostapd_configured'] = FALSE;

        if (file_exists($hostapd_config)) {
            $output = shell_exec("cat $hostapd_config | grep $device" );
            if (isset($output)){
                $config_file = new Configuration_File($hostapd_config);
                $hostapd_settings =  $config_file->load();
                $hostapd_settings['hostapd_configured'] = TRUE;
            } else {
                $hostapd_settings['hostapd_configured'] = FALSE;
            }
        }
      
        //Rename channel.  For Display purposes only 
        if( $hostapd_settings['channel'] == 'acs' )
            $hostapd_settings['channel'] == 'Automatic';


        return $hostapd_settings;
    }

    public function get_default_hostapd_settings($device){
        clearos_profile(__METHOD__, __LINE__);

        $hostapd['interface']                       = '$device';
        $hostapd['driver']                          = 'nl80211';
        $hostapd['ctrl_interface']                  = '/var/run/hostapd';
        $hostapd['ctrl_interface_group']            = '0';
        $hostapd['wds_sta']                         = '1'; //allows associated stations to use 4-address frames to allow layer 2
        $hostapd['ssid']                            = 'Hub-Machine';
        $hostapd['hw_mode']                         = 'g';
        $hostapd['channel']                         = 'Automatic'; // Actual value is acs, set to  we will change this to acs when saving
        $hostapd['max_num_sta']                     = '128';
        $hostapd['disassoc_low_ack']                = '1';
        //$hostapd['auth_algs']                       = '1'; // 1=Only allow WPA2 , wpa 1 disabled . should only be used with open systems.
        $hostapd['wpa']                             = '2'; //1=wpa, 2=wep, 3=both
        $hostapd['wpa_key_mgmt']                    = 'WPA-PSK'; // Doesn't hurt to add them all
        $hostapd['wpa_passphrase']                  =  $this->random_password(8);
        $hostapd['wpa_pairwise']                    = 'CCMP';  // Key used for WPA 1. TKIP CCMP CCMP-256 GCMP GCMP-256. Most systems don't support CCMP-256 or GMCP-256
        //$hostapd['rsn_pairwise']                    = 'CCMP';  // Key used for WPA 2. If left unset, it will automatically use the wpa_pairwise ciphers.
        $hostapd['country_code']                    = 'US';
        $hostapd['ieee80211d']                      = '1' ; // limit the frequencies to country code ( always =1 )
        $hostapd['ieee80211h']                      = '0'; // Enable Radar Detection. You need to recompile the kernel with DFS Support for this to work. 
        $hostapd['acs_exclude_dfs']                 = '1';
        $hostapd['spectrum_mgmt_required']          = '0';
        $hostapd['ieee80211n']                      = '0';
        //$hostapd['']                              = '0';
        //$hostapd['require_vht']                   = '0';
        $hostapd['ieee80211ac']                     = '1';
        $hostapd['vht_oper_chwidth']                = '0';
        $hostapd['use_sta_nsts']                    = '1';
        $hostapd['uapsd_advertisement_enabled']     = '1';
        $hostapd['wmm_enabled']                     = '1';

        return $hostapd;

    }

    public function get_dfs_status(){
        clearos_profile(__METHOD__, __LINE__);

        $dfs= FALSE;
        $output = shell_exec(" dmesg | grep dfs ");
        if (strpos($output,"dfs 1"))
            $dfs=TRUE;

        return $dfs;
    }
 
    public function get_device_iw_info($device){
        clearos_profile(__METHOD__, __LINE__);

        $physical_id = shell_exec(`iw $device info | grep -m1 -oP '(?<=wiphy ).*' `);
        $physical_id = $physical_id / 1;  //sanity check , plus convert into an int. 
        $iw_info =  shell_exec(" iw phy" . $physical_id ." info ");

        return $iw_info;
    }

    public function get_device_channels($iw_info){
        clearos_profile(__METHOD__, __LINE__);
        
        if (strpos($iw_info,"HT40")){
            $channels['g']['Automatic']     = 'Automatic';
            $channels['a']['Automatic']     = 'Automatic';
            $channels['dfs']['Automatic']   = 'Automatic';
        }
        
        for ($i = 1; $i <= 14; $i++){

            if (strpos($iw_info,"[$i]")){  

                $get_channel=$this->getBetween($iw_info ,"[$i]","\n");
                
                if(strpos($get_channel," dBm)"))
                    $channels['g'][$i]  = $i;
            }
        }

        for ($i = 36; $i <= 169; $i++){

            if (strpos($iw_info,"[$i]")){
               
                $get_channel=$this->getBetween($iw_info ,"[$i]","\n");
                
                if(strpos($get_channel," dBm)") && !strpos($get_channel,"no IR") ) {
                    $channels['a'][$i]  = $i;
                    $channels['dfs'][$i]   = $i;
                }
                    
                if( strpos($get_channel,"no IR") && strpos($get_channel,"radar detection"))
                    $channels['dfs'][$i]   = $i;
            }
        }
        
        if( !isset($channels['g']) )
            $channels['g']['Automatic']     =  lang('wireless_ap_no_channels');
        
        if( !isset($channels['a'] ) )
            $channels['a']['Automatic']     =  lang('wireless_ap_no_channels');
 
        if( !isset($channels['dfs'] ) )
            $channels['dfs']['Automatic']   =  lang('wireless_ap_no_channels');

        return $channels;
    }

    public function get_device_general_info($iw_info){
        clearos_profile(__METHOD__, __LINE__);

        $iw_info =  $this->getBetween($iw_info ,"Wiphy","Band");

        if (strpos($iw_info_general,"WEP40")) 
            $general['WEP40'] = TRUE;

        if (strpos($iw_info_general,"WEP140")) 
            $general['WEP140'] = TRUE;
            
        if (strpos($iw_info_general,"TKIP"))  
            $general['TKIP'] = TRUE;

        if (strpos($iw_info_general,"CCMP")) 
            $general['CCMP'] = TRUE;

        if (strpos($iw_info_general,"CMAC"))  
            $general['CMAC'] = TRUE;

        if (strpos($iw_info_general,"GCMP")) 
            $general['GCMP'] = TRUE;

        if (strpos($iw_info_general,"* IBSS")) 
            $general['IBSS'] = TRUE;

        if (strpos($iw_info_general,"* managed"))  
            $general['managed'] = TRUE;

        if (strpos($iw_info_general,"* AP")) 
            $general['AP'] = TRUE;

        if (strpos($iw_info_general,"* AP/VLAN")) 
            $general['AP/VLAN'] = TRUE;

        if (strpos($iw_info_general,"* monitor")) 
            $general['monitor'] = TRUE;

        if (strpos($iw_info_general,"* mesh point")) 
            $general['mesh'] = TRUE;
        
        return $general_info;
    }

    public function get_hostapd_options($iw_info){
        clearos_profile(__METHOD__, __LINE__);


        $options['bandwidth_g'] = array (
            20 => '20 Mhz',
        );

        $options['bandwidth_a'] = array (
            20 => '20 Mhz',
        );

        $options['hw_mode'] = [];
        if (strpos($iw_info,"5260 MHz") && strpos($iw_info,"2412 MHz")){

            $options['hw_mode'] = array( 
                a => '5.0 GHZ',
                g => '2.4 GHZ',
            );

        } elseif ( strpos($iw_info,"5260 MHz") ){ 
            $options['hw_mode'] = array( 
                a => '5.0 GHZ',
            );
  
        } elseif ( strpos($iw_info,"2412 MHz") ){
            $options['hw_mode'] = array(
                 g => '2.4 GHZ',
            );
            
        }

        $options['country'] = array (
              'AD' => 'AD',
            'AE' => 'AE',
            'AF' => 'AF',
            'AI' => 'AI',
            'AL' => 'AL',
            'AM' => 'AM',
            'AN' => 'AN',
            'AR' => 'AR',
            'AS' => 'AS',
            'AT' => 'AT',
            'AU' => 'AU',
            'AW' => 'AW',
            'AZ' => 'AZ',
            'BA' => 'BA',
            'BB' => 'BB',
            'BD' => 'BD',
            'BE' => 'BE',
            'BF' => 'BF',
            'BG' => 'BG',
            'BH' => 'BH',
            'BL' => 'BL',
            'BM' => 'BM',
            'BN' => 'BN',
            'BO' => 'BO',
            'BR' => 'BR',
            'BS' => 'BS',
            'BT' => 'BT',
            'BY' => 'BY',
            'BZ' => 'BZ',
            'CA' => 'CA',
            'CF' => 'CF',
            'CH' => 'CH',
            'CI' => 'CI',
            'CL' => 'CL',
            'CN' => 'CN',
            'CO' => 'CO',
            'CR' => 'CR',
            'CU' => 'CU',
            'CX' => 'CX',
            'CY' => 'CY',
            'CZ' => 'CZ',
            'DE' => 'DE',
            'DK' => 'DK',
            'DM' => 'DM',
            'DO' => 'DO',
            'DZ' => 'DZ',
            'EC' => 'EC',
            'EE' => 'EE',
            'EG' => 'EG',
            'ES' => 'ES',
            'ET' => 'ET',
            'FI' => 'FI',
            'FM' => 'FM',
            'FR' => 'FR',
            'GB' => 'GB',
            'GD' => 'GD',
            'GE' => 'GE',
            'GF' => 'GF',
            'GH' => 'GH',
            'GL' => 'GL',
            'GP' => 'GP',
            'GR' => 'GR',
            'GT' => 'GT',
            'GU' => 'GU',
            'GY' => 'GY',
            'HK' => 'HK',
            'HN' => 'HN',
            'HR' => 'HR',
            'HT' => 'HT',
            'HU' => 'HU',
            'ID' => 'ID',
            'IE' => 'IE',
            'IL' => 'IL',
            'IN' => 'IN',
            'IR' => 'IR',
            'IS' => 'IS',
            'IT' => 'IT',
            'JM' => 'JM',
            'JO' => 'JO',
            'JP' => 'JP',
            'KE' => 'KE',
            'KH' => 'KH',
            'KN' => 'KN',
            'KP' => 'KP',
            'KR' => 'KR',
            'KW' => 'KW',
            'KY' => 'KY',
            'KZ' => 'KZ',
            'LB' => 'LB',
            'LC' => 'LC',
            'LI' => 'LI',
            'LK' => 'LK',
            'LS' => 'LS',
            'LT' => 'LT',
            'LU' => 'LU',
            'LV' => 'LV',
            'MA' => 'MA',
            'MC' => 'MC',
            'MD' => 'MD',
            'ME' => 'ME',
            'MF' => 'MF',
            'MH' => 'MH',
            'MK' => 'MK',
            'MN' => 'MN',
            'MO' => 'MO',
            'MP' => 'MP',
            'MQ' => 'MQ',
            'MR' => 'MR',
            'MT' => 'MT',
            'MU' => 'MU',
            'MV' => 'MV',
            'MW' => 'MW',
            'MX' => 'MX',
            'MY' => 'MY',
            'NG' => 'NG',
            'NI' => 'NI',
            'NL' => 'NL',
            'NO' => 'NO',
            'NP' => 'NP',
            'NZ' => 'NZ',
            'OM' => 'OM',
            'PA' => 'PA',
            'PE' => 'PE',
            'PF' => 'PF',
            'PG' => 'PG',
            'PH' => 'PH',
            'PK' => 'PK',
            'PL' => 'PL',
            'PM' => 'PM',
            'PR' => 'PR',
            'PT' => 'PT',
            'PW' => 'PW',
            'PY' => 'PY',
            'QA' => 'QA',
            'RE' => 'RE',
            'RO' => 'RO',
            'RS' => 'RS',
            'RU' => 'RU',
            'RW' => 'RW',
            'SA' => 'SA',
            'SE' => 'SE',
            'SG' => 'SG',
            'SI' => 'SI',
            'SK' => 'SK',
            'SN' => 'SN',
            'SR' => 'SR',
            'SV' => 'SV',
            'SY' => 'SY',
            'TC' => 'TC',
            'TD' => 'TD',
            'TG' => 'TG',
            'TH' => 'TH',
            'TN' => 'TN',
            'TR' => 'TR',
            'TT' => 'TT',
            'TW' => 'TW',
            'TZ' => 'TZ',
            'UA' => 'UA',
            'UG' => 'UG',
            'US' => 'US',
            'UY' => 'UY',
            'UZ' => 'UZ',
            'VC' => 'VC',
            'VE' => 'VE',
            'VI' => 'VI',
            'VN' => 'VN',
            'VU' => 'VU',
            'WF' => 'WF',
            'WS' => 'WS',
            'YE' => 'YE',
            'YT' => 'YT',
            'ZA' => 'ZA',
            'ZW' => 'ZW',

        );

        $options['wpa'] = array (
            '3' => 'WPA / WPA2 - TKIP CCMP',
            '2' => 'WPA2 - CCMP ',
        );

        $options['vht_oper_chwidth'] = array (
            '0' => 'Disabled',
            '1' => 'Enabled',
        );

        $options['acs_exclude_dfs'] = array (
            '0' => 'Enabled',
            '1' => 'Disabled',
        );

        if (strpos($iw_info,"HT40")){
            $options['ieee80211n'] = array (
                '0' => 'Disabled',
                '1' => 'Enabled',
            );
            
            $options['bandwidth_g'] += [ 40 => '40 MHz' ];
            $options['bandwidth_a'] += [ 40 => '40 MHz' ];
           
        } else {
            $options['ieee80211n'] = FALSE;
        }

        if (strpos($iw_info,"VHT Capabilities")){
            $options['ieee80211ac'] = array (
                '0' => 'Disabled',
                '1' => 'Enabled',
            );
            $options['bandwidth_a'] += [ 80 => '80 MHz' ];
        } else {
            $options['ieee80211ac'] = FALSE;
        }
        
        $options['uapsd_advertisement_enabled'] = array (
            '0' => 'Disabled',
            '1' => 'Enabled',
        );

        $options['spectrum_mgmt_required'] = array (
            '0' => 'Disabled',
            '1' => 'Enabled',
        );

         // wwm requires 802.11n 
        if ($options['ieee80211n'] == FALSE ){
            $options['wmm_enabled'] = array (
                '0' => 'Disabled, Not Available',
            );
        } else {
            $options['wmm_enabled'] = array (
                '0' => 'Disabled',
                '1' => 'Enabled',
            );
        }
        

        $options['disassoc_low_ack'] = array (
            '0' => 'Disabled',
            '1' => 'Enabled',
        );

        

        return $options;
    }

    public function get_ht_capab($iw_info,$hw_mode){
        clearos_profile(__METHOD__, __LINE__);
        // Band 1 options  
        // need to add option to test how many bands are on wifi adapter. 
        $iw_info = $this->getBetween($iw_info ,"Band 1:","2484");
         
        // missing options
        // [GF][ht40-][LSIG-TXOP-PROT]

        if (strpos($iw_info,"RX LDPC"))
            $ht_capab='[LDPC]';

        #freq		    HT40-		    HT40+
        #	2.4 GHz		5-13		    1-7 (1-9 in Europe/Japan)
        #	5 GHz		40,48,56,64	    36,44,52,60

        if (strpos($iw_info,"HT40")) 
            $ht_capab ='[HT40+]'. $ht_capab ;
            
        //need to test 
        if (strpos($iw_info,"SM Power Save static")) 
            $ht_capab ='[SMPS-STATIC]'. $ht_capab ;

        //need to test
        if (strpos($iw_info,"SM Power Save dynamic")) 
            $ht_capab ='[SMPS-DYNAMIC]'. $ht_capab;

        if (strpos($iw_info,"RX HT20 SGI")) 
            $ht_capab ='[SHORT-GI-20]'. $ht_capab;

        if (strpos($iw_info,"RX HT40 SGI")) 
            $ht_capab ='[SHORT-GI-40]'. $ht_capab ;
            
        if (strpos($iw_info,"TX STBC")) 
            $ht_capab ='[TX-STBC]'. $ht_capab ;

        if (strpos($iw_info,"RX STBC 1-stream")) 
            $ht_capab ='[RX-STBC1]'. $ht_capab ;

        //needs testing
        if (strpos($iw_info,"RX STBC 2-stream")) 
            $ht_capab ='[RX-STBC12]'. $ht_capab ;

        //needs testing
        if (strpos($iw_info,"RX STBC 3-stream")) 
            $ht_capab ='[RX-STBC123]'. $ht_capab ;

        if (strpos($iw_info,"Max AMSDU length: ")){
            $temp=$this->getBetween($iw_info ,"Max AMSDU length:","bytes");
            if(strpos($temp,"7935"))
                $ht_capab ='[MAX-AMSDU-7935]'. $ht_capab ;
        }

        if (strpos($iw_info,"DSSS/CCK HT40")) 
            $ht_capab ='[DSSS_CCK-40]'. $ht_capab;

        return $ht_capab;
    }

    public function get_vht_capab($iw_info){
        clearos_profile(__METHOD__, __LINE__);
        
        // missing config options.   My wifi adpater doesn't support these options. no way for me to test. 
        // [VHT160] [VHT160-80PLUS80] [SHORT-GI-160]
        // [SU-BEAMFORMER] [SU-BEAMFORMEE]
        // [BF-ANTENNA-2] [BF-ANTENNA-3] [BF-ANTENNA-4]
        // [SOUNDING-DIMENSION-2] [SOUNDING-DIMENSION-3] [SOUNDING-DIMENSION-4]
        // [MU-BEAMFORMER]
        // [VHT-TXOP-PS]  [HTC-VHT]
        // [MAX-A-MPDU-LEN-EXP0]..[MAX-A-MPDU-LEN-EXP7]
        // [VHT-LINK-ADAPT2] [VHT-LINK-ADAPT3]
        // [HTC-VHT] 
        // [RX-ANTENNA-PATTERN][TX-ANTENNA-PATTERN]
       
        $iw_info= $this->getBetween($iw_info ,"Band 2:","5865");

        $vht_capab = '[MAX-A-MPDU-LEN-EXP3]';

        if (strpos($iw_info,"RX LDPC"))
            $vht_capab='[RXLDPC]' . $vht_capab;

        if (strpos($iw_info,"short GI (80 MHz)"))
            $vht_capab='[SHORT-GI-80]' . $vht_capab;

        if (strpos($iw_info,"Max MPDU length:")){
            $temp=$this->getBetween($iw_info ,"Max MPDU length:","\n");
            if(strpos($temp,"11454"))
                $vht_capab ='[MAX-MPDU-11454]'. $vht_capab ;
            if(strpos($temp,"7991"))
                $vht_capab ='[MAX-MPDU-7991]'. $vht_capab ;  
        }

        if (strpos($iw_info,"VHT TX MCS set:")){
            $temp=$this->getBetween($iw_info ,"VHT TX MCS set:","VHT TX highest");
            if(strpos($temp,"2 streams: MCS"))
                $vht_capab='[TX-STBC-2BY1]' . $vht_capab;
        }
        
        if (strpos($iw_info,"VHT RX MCS set:")){
            $temp=$this->getBetween($iw_info ,"RX STBC","\n");
            if(strpos($temp,"4-stream")){
                $vht_capab='[RX-STBC-1234]' . $vht_capab;
            } elseif (strpos($temp,"3-stream")){
                $vht_capab='[RX-STBC-123]' . $vht_capab; 
            } elseif (strpos($temp,"2-stream")){
                $vht_capab='[RX-STBC-12]' . $vht_capab;
            } elseif (strpos($temp,"1-stream")){
                $vht_capab='[RX-STBC-1]' . $vht_capab;
            } 

        }
        
        if (strpos($iw_info,"RX antenna pattern consistency"))
            $vht_capab='[RX-ANTENNA-PATTERN]' . $vht_capab;
        
        if (strpos($iw_info,"TX antenna pattern consistency"))
            $vht_capab='[TX-ANTENNA-PATTERN]' . $vht_capab;
        
        return $vht_capab;
    }
    
    public function set_hostapd($hostapd,$iw_info){
        clearos_profile(__METHOD__, __LINE__); 

        
        //Sanity Checks
        if( $hostapd['ieee80211n'] === '0' )
            $hostapd['vht_oper_chwidth'] = '0';

        if ($hostapd['acs_exclude_dfs'] == NULL )
            $hostapd['acs_exclude_dfs'] == '1';    


        // for automatic channel selection, build a list of what chanels hostapd can and cannot use. 
        // start by creating a list of all the channels. 

        //remove uneeded variableds from hostapd 
        $remove_from_hostapd  =  [];


        // Set Center Ferquency if  channle != Automatic
         
        /////////////////////////////////////////////
        // Channel Selection 
        // Get channel selection and calculate the 
        // center channel frequency if necessary 
        ////////////////////////////////////////////

        if( $hostapd['hw_mode'] === 'g'){
    
            $hostapd['channel'] = $hostapd['channel_g'];
            array_push($remove_from_hostapd, 'ieee80211h', 'acs_exclude_dfs','spectrum_mgmt_required','local_pwr_constraint','ieee80211ac','vht_oper_chwidth');
    
        } else if( $hostapd['hw_mode'] === 'a' && $hostapd['acs_exclude_dfs'] === '0') {

            $hostapd['channel'] = $hostapd['channel_dfs'];

        } else if( $hostapd['hw_mode'] === 'a') {

            $hostapd['channel'] = $hostapd['channel_a'];

        }

        if( $hostapd['channel'] == 'Automatic'){

            $hostapd['channel'] = 'acs';
            array_push($remove_from_hostapd,'vht_oper_centr_freq_seg0_idx');

        } else {

            if( $hostapd['ieee80211n'] === '1'	&	$hostapd['vht_oper_chwidth'] === '1' ){
                
                $hostapd['vht_oper_centr_freq_seg0_idx'] = $hostapd['channel']  + 6 ;

            } else if( $hostapd['ieee80211n'] === '1'	&	$hostapd['vht_oper_chwidth'] === '0' ){

                $hostapd['vht_oper_centr_freq_seg0_idx'] = $hostapd['channel']  + 2 ;

            } else if( $hostapd['ieee80211n'] === '0'	&	$hostapd['vht_oper_chwidth'] === '0' ){

                array_push($remove_from_hostapd,'vht_oper_centr_freq_seg0_idx');

            }
        
        }
        /////////////////////////////////////////////
        // END Channel Selection 
        ////////////////////////////////////////////

        
        /////////////////////////////////////////////
        // Automatic Channel Selection List
        ////////////////////////////////////////////
        if ( $hostapd['channel'] === 'acs'){
            $remove_from_chanlist =  [];

            // Get the list of channels. 
            $chanlist['1']      = $hostapd['g']['001'];   
            $chanlist['2']      = $hostapd['g']['002'];   
            $chanlist['3']      = $hostapd['g']['003'];   
            $chanlist['4']      = $hostapd['g']['004'];   
            $chanlist['5']      = $hostapd['g']['005'];   
            $chanlist['6']      = $hostapd['g']['006'];   
            $chanlist['7']      = $hostapd['g']['007'];   
            $chanlist['8']      = $hostapd['g']['008'];   
            $chanlist['9']      = $hostapd['g']['009'];   
            $chanlist['10']     = $hostapd['g']['010'];   
            $chanlist['11']     = $hostapd['g']['011']; 
            $chanlist['12']     = $hostapd['g']['012'];
            $chanlist['13']     = $hostapd['g']['013']; 
            $chanlist['14']     = $hostapd['g']['014'];
            
            if ( $hostapd['acs_exclude_dfs'] === '0' ){
                $chanlist['36']     = $hostapd['dfs']['036'];
                $chanlist['40']     = $hostapd['dfs']['040'];
                $chanlist['44']     = $hostapd['dfs']['044'];
                $chanlist['48']     = $hostapd['dfs']['048'];
                $chanlist['52']     = $hostapd['dfs']['052'];
                $chanlist['56']     = $hostapd['dfs']['056'];
                $chanlist['60']     = $hostapd['dfs']['060'];
                $chanlist['64']     = $hostapd['dfs']['064'];
                $chanlist['100']    = $hostapd['dfs']['100'];
                $chanlist['104']    = $hostapd['dfs']['104'];
                $chanlist['108']    = $hostapd['dfs']['108'];
                $chanlist['112']    = $hostapd['dfs']['112'];
                $chanlist['116']    = $hostapd['dfs']['116'];
                $chanlist['120']    = $hostapd['dfs']['120'];
                $chanlist['124']    = $hostapd['dfs']['124'];
                $chanlist['128']    = $hostapd['dfs']['128'];
                $chanlist['132']    = $hostapd['dfs']['132'];
                $chanlist['136']    = $hostapd['dfs']['136'];
                $chanlist['140']    = $hostapd['dfs']['140'];
                $chanlist['144']    = $hostapd['dfs']['144'];
                $chanlist['149']    = $hostapd['dfs']['149'];
                $chanlist['153']    = $hostapd['dfs']['153'];
                $chanlist['157']    = $hostapd['dfs']['157'];
                $chanlist['161']    = $hostapd['dfs']['161'];
                $chanlist['165']    = $hostapd['dfs']['165'];
            } else {
                $chanlist['36']     = $hostapd['a']['036'];
                $chanlist['40']     = $hostapd['a']['040'];
                $chanlist['44']     = $hostapd['a']['044'];
                $chanlist['48']     = $hostapd['a']['048'];
                $chanlist['52']     = $hostapd['a']['052'];
                $chanlist['56']     = $hostapd['a']['056'];
                $chanlist['60']     = $hostapd['a']['060'];
                $chanlist['64']     = $hostapd['a']['064'];
                $chanlist['100']    = $hostapd['a']['100'];
                $chanlist['104']    = $hostapd['a']['104'];
                $chanlist['108']    = $hostapd['a']['108'];
                $chanlist['112']    = $hostapd['a']['112'];
                $chanlist['116']    = $hostapd['a']['116'];
                $chanlist['120']    = $hostapd['a']['120'];
                $chanlist['124']    = $hostapd['a']['124'];
                $chanlist['128']    = $hostapd['a']['128'];
                $chanlist['132']    = $hostapd['a']['132'];
                $chanlist['136']    = $hostapd['a']['136'];
                $chanlist['140']    = $hostapd['a']['140'];
                $chanlist['144']    = $hostapd['a']['144'];
                $chanlist['149']    = $hostapd['a']['149'];
                $chanlist['153']    = $hostapd['a']['153'];
                $chanlist['157']    = $hostapd['a']['157'];
                $chanlist['161']    = $hostapd['a']['161'];
                $chanlist['165']    = $hostapd['a']['165'];
            }

            if( $hostapd['hw_mode'] === 'g'){

                // These options are not valid for 2.4 Ghz 
        
                array_push($remove_from_chanlist,'36','40','44','48','52','56','60','64','100','104','108','112');
                array_push($remove_from_chanlist,'116','120','124','128','132','136','140','144','149','153','157','161','165');
                
                if( $hostapd['ieee80211n'] === '1')
                    array_push($remove_from_chanlist,'10','11','12','13');

            } else if( $hostapd['hw_mode'] === 'a') {

                array_push($remove_from_chanlist,'1','2','3','4','5','6','7','8','9','10','11','12','13');

                if( $hostapd['ieee80211n'] === '1')
                    array_push($remove_from_chanlist,'40','48','56','64','104','112','120','128','136','144','153','161','165');
                
                if( $hostapd['vht_oper_chwidth'] === '1' )
                    array_push($remove_from_chanlist,'44','60','108','124','140','157');

                if( $hostapd['acs_exclude_dfs'] === '1' )
                    array_push($remove_from_chanlist,'52','56','60','64','100','104','108','112','116','120','124','128','132','136','140','144');
            }

            // Build Chanel list and add it to the hostapd array 
            foreach($remove_from_chanlist as $key) {
                    unset($chanlist[$key]);
            }

            foreach ($chanlist as $key => $value) {
                if($value > '0' ){
                    $final_chanlist = $key. ' '. $final_chanlist ;
                }        
            }

            $hostapd['chanlist']= $final_chanlist;
        }
        /////////////////////////////////////////////
        // End Automatic Channel Selection List
        ////////////////////////////////////////////

        // Remove uneeded keys from the hostapd arrary 
        array_push($remove_from_hostapd, 'g' , 'a' , 'dfs' , 'channel_g' ,'channel_a' , 'channel_dfs' , 'hostapd_configured');

        foreach($remove_from_hostapd as $key) {
                unset($hostapd[$key]);
        }

        $file = new File( self::FILE_HOSTAPD_CONFIG . $hostapd['interface'] . '.conf');
        if ($file->exists())
            $file->delete();
        
        //  set country code 
        shell_exec( self::COMMAND_IW . " reg set " . $hostapd['country']);
        
        $file->create('root', 'root', '0644');

        foreach ($hostapd as $key => $value) {
                if(isset($value)) 
                    $file->add_lines($key . '=' . $value . "\n");
        }
        
        
        
         return TRUE;
    }

    public function validate_ssid($ssid){
        clearos_profile(__METHOD__, __LINE__);
        /* 
        The length of the SSID field is between 0 and 32 octets. A SSID field of length 0 is used within Probe
        Request management frames to indicate the wildcard SSID. The wildcard SSID is also used in Beacon and
        Probe Response frames transmitted by mesh STAs.
            When the UTF-8 SSID subfield of the Extended Capabilities element is equal to 1 in the frame that includes
        the SSID element, the SSID is interpreted using UTF-8 encoding. 
        */

        if (!preg_match("/^[\\x00-\\x7F]{0,32}$/u", $ssid))
            return lang('wireless_ap_ssid_invalid');

    }

    public function validate_wpa_passphrase($passphrase){
        clearos_profile(__METHOD__, __LINE__);

        // REGEX match for printable ASCII characters 8-63 
        if (!preg_match("/^[\\x00-\\x7F]{8,63}$/u", $passphrase))
            return lang('wireless_ap_passphrase_invalid');
    }

    public function delete_interface($device) {
        
        clearos_profile(__METHOD__, __LINE__);
        
        $file = new File(self::FILE_HOSTAPD_CONFIG . $device . '.conf');
        $file->delete();
    }
}
